package nameana;
import java.util.ArrayList;

import ast.Referenceable;

public class SymbolTable {
	private static class ScopeEntry {
		public Symbol symbol;
		public Referenceable oldDefinition;
		public ScopeEntry(Symbol symbol, Referenceable oldDefinition) {
			this.symbol = symbol;
			this.oldDefinition = oldDefinition;
		}
	}
	static class Scope {
		public Scope parent;
		public ArrayList<ScopeEntry> entries = new ArrayList<ScopeEntry>();
		public Scope(Scope parent) {
			this.parent = parent;
		}
	}

	private Scope currentScope = new Scope(null);

	/**
	 * opens a new scope inside the current scope
	 */
	public void enterScope() {
		currentScope = new Scope(currentScope);
	}

	/**
	 * leave scope entered by enterScope.
	 * You must leave scopes in exactly the reverse order you entered them!
	 */
	public void leaveScope() {
		Scope parent = currentScope.parent;
		for (ScopeEntry entry : currentScope.entries)
			entry.symbol.setCurrentDefinition(parent, entry.oldDefinition);
		currentScope = parent;
	}

	/**
	 * set Definition of a symbol in the current scope
	 */
	 public void enterDefinition(Symbol symbol, Referenceable definition) {
		 Referenceable oldDefinition = symbol.getCurrentDefinition();
		 ScopeEntry entry = new ScopeEntry(symbol, oldDefinition);
		 currentScope.entries.add(entry);
		 symbol.setCurrentDefinition(currentScope, definition);
	 }

	 /**
	  * return current definition of a symbol (the innermost scope defining it)
	  */
	 public Referenceable getCurrentDefinition(Symbol symbol) {
		 return symbol.getCurrentDefinition();
	 }

	 /**
	  * returns true if symbol has a definition in the current scope.
	  * (Typically used to detect double definitions)
	  */
	 public boolean definitionInCurrentScope(Symbol symbol) {
		 return symbol.getCurrentScope() == currentScope;
	 }
}